#include <common.h>
#include <nand.h>

struct fis_image_desc {
    char name[16];               // Null terminated name
    uint32_t      flash_base;    // Address within FLASH of image
    uint32_t      mem_base;      // Address in memory where it executes
    uint32_t      size;          // Length of image
    uint32_t      entry_point;   // Execution entry point
    uint32_t      data_length;   // Length of actual data
    unsigned char _pad[256-(16+7*sizeof(uint32_t))];
    uint32_t      desc_cksum;    // Checksum over image descriptor
    uint32_t      file_cksum;    // Checksum over image data
};

#define MaxEntries 16
static struct fis_image_desc  FIStable[MaxEntries];
static char mtdparts[1024];

void parse_redboot_FIS_table(void)
{
  unsigned i,n=0;
  struct fis_image_desc *d;
  size_t len = sizeof FIStable;
  
  if (nand_read_skip_bad(&nand_info[0], CONFIG_REDBOOT_PART_OFFSET, &len, (uchar*)&FIStable))
    {
      printf("Read RedBoot FIS table from NAND at 0x%x failed\n",CONFIG_REDBOOT_PART_OFFSET);
      return;
    }

  for (i=0; i<MaxEntries; ++i)
    {
      if (!strcmp("FIS directory",FIStable[i].name))
        break;
    }

  if (i == MaxEntries)
    {
      printf("RedBoot FIS table not found\n");
      return;
    }

  setenv("mtdids","nand0=atmel_nand");

  printf("Found RedBoot FIS table at 0x%0x\n",CONFIG_REDBOOT_PART_OFFSET);

  n += sprintf(mtdparts+n,"mtdparts=atmel_nand:");

  for (d=FIStable, i=0; i<MaxEntries; ++i,++d)
    {
      char *sep = i ? "," : "";
      if (d->name[0] == 0xff && d->name[0] == 0xff)
        break;
      n += sprintf(mtdparts+n,"%s0x%x@0x%x(%s)", sep, d->size, d->flash_base, d->name);
      printf("%18s at %08x size %08x\n",d->name,d->flash_base,d->size);
    }

  setenv("mtdparts",mtdparts);
}
